@@ -13,18 +13,33 @@ import sys
 from wheatley.rhythm import RegressionRhythm, WaitForUserRhythm
 from wheatley.tower import RingingRoomTower
 from wheatley.bot import Bot
-from wheatley.page_parser import get_load_balancing_url
+from wheatley.page_parser import get_load_balancing_url, TowerNotFoundError, InvalidURLError
+from wheatley.arg_parsing import parse_peal_speed, PealSpeedParseError, parse_call
+
 from wheatley.row_generation import RowGenerator, ComplibCompositionGenerator
 from wheatley.row_generation import MethodPlaceNotationGenerator
+from wheatley.row_generation.complib_composition_generator import PrivateCompError, InvalidCompError
+from wheatley.row_generation.method_place_notation_generator import MethodNotFoundError, generator_from_special_title
 
 
 def row_generator(args):
     """ Generates a row generator according to the given CLI arguments. """
 
     if "comp" in args and args.comp is not None:
-        row_gen = ComplibCompositionGenerator(args.comp)
+        try:
+            row_gen = ComplibCompositionGenerator(args.comp)
+        except (PrivateCompError, InvalidCompError) as e:
+            sys.exit(f"Bad value for '--comp': {e}")
     elif "method" in args:
-        row_gen = MethodPlaceNotationGenerator(args.method)
+        try:
+            row_gen = generator_from_special_title(args.method) or \
+                      MethodPlaceNotationGenerator(
+                          args.method,
+                          parse_call(args.bob),
+                          parse_call(args.single)
+                      )
+        except MethodNotFoundError as e:
+            sys.exit(f"Bad value for '--method': {e}")
     else:
         assert False, \
             "This shouldn't be possible because one of --method and --comp should always be defined"
@@ -37,91 +52,6 @@ def row_generator(args):
     return row_gen
 
 
-class PealSpeedParseError(ValueError):
-    """
-    An error thrown with a helpful error message when the user inputs a peal speed string that
-    cannot be parsed by the parser.
-    """
-
-    def __init__(self, peal_spead_string, message):
-        super().__init__()
-
-        self.peal_spead_string = peal_spead_string
-        self.message = message
-
-    def __str__(self):
-        return f"Error parsing peal speed '{self.peal_spead_string}': {self.message}"
-
-
-def parse_peal_speed(peal_speed: str):
-    """
-    Parses a peal speed written in the format /2h58(m?)/ or /XXX(m?)/ into a number of minutes.
-    """
-
-    def exit_with_message(error_text):
-        """ Raise an exception with a useful error message. """
-
-        raise PealSpeedParseError(peal_speed, error_text)
-
-    # Strip whitspace from the argument, so that if the user is in fact insane enough to pad their
-    # CLI arguments with whitespace then they can do so and not crash the program.  This also has
-    # the side effect of cloning the input string so we can freely modify it.
-    stripped_peal_speed = peal_speed.strip()
-
-    # Remove the 'm' from the end of the peal speed - it doesn't add any clarity
-    if stripped_peal_speed.endswith("m"):
-        stripped_peal_speed = stripped_peal_speed[:-1]
-
-    if "h" in stripped_peal_speed:
-        # Split the peal speed into its hour and minute components, and print a helpful message
-        # if there are too many parts
-        split_parts = stripped_peal_speed.split("h")
-
-        if len(split_parts) > 2:
-            exit_with_message("The peal speed should contain at most one 'h'.")
-
-        hour_string, minute_string = split_parts
-
-        # Strip the input values so that the user can put whitespace into the input if they want
-        hour_string = hour_string.strip()
-        minute_string = minute_string.strip()
-
-        # Parse the hours value, and print messages if it is invalid
-        try:
-            hours = int(hour_string)
-        except ValueError:
-            exit_with_message(f"The hour value '{hour_string}' is not an integer.")
-
-        if hours < 0:
-            exit_with_message(f"The hour value '{hour_string}' must be a positive integer.")
-
-        # Parse the minute value, and print messages if it is invalid
-        try:
-            minutes = 0 if minute_string == "" else int(minute_string)
-        except ValueError:
-            exit_with_message(f"The minute value '{minute_string}' is not an integer.")
-
-        if minutes < 0:
-            exit_with_message(f"The minute value '{minute_string}' must be a positive integer.")
-
-        if minutes > 59:
-            exit_with_message(f"The minute value '{minute_string}' must be smaller than 60.")
-
-        return hours * 60 + minutes
-
-    # If the user doesn't put an 'h' in their string, then we assume it's just a minute value that
-    # may or may not be bigger than 60
-    try:
-        minutes = int(stripped_peal_speed)
-    except ValueError:
-        exit_with_message(f"The minute value '{stripped_peal_speed}' is not an integer.")
-
-    if minutes < 0:
-        exit_with_message(f"The minute value '{stripped_peal_speed}' must be a positive integer.")
-
-    return minutes
-
-
 def rhythm(args):
     """ Generates a rhythm object according to the given CLI arguments. """
 
@@ -137,7 +67,7 @@ def rhythm(args):
         max_rows_in_dataset=args.max_rows_in_dataset
     )
 
-    if args.wait:
+    if not args.keep_going:
         return WaitForUserRhythm(regression)
 
     return regression
@@ -215,9 +145,10 @@ def main():
 
     # Rhythm arguments
     parser.add_argument(
-        "-w", "--wait",
+        "-k", "--keep-going",
         action="store_true",
-        help="If set, the bot will wait for users to ring rather than pushing on with the rhythm."
+        help="If set, the bot will not wait for users to ring - instead, it will push on with the \
+              rhythm."
     )
     parser.add_argument(
         "-I", "--inertia",
@@ -258,22 +189,51 @@ def main():
     # Row generator arguments
     row_gen_group = parser.add_mutually_exclusive_group(required=True)
     row_gen_group.add_argument(
-        "--comp",
+        "-c", "--comp",
         type=int,
         help="The ID of the complib composition you want to ring"
     )
     row_gen_group.add_argument(
-        "--method",
+        "-m", "--method",
         type=str,
         help="The title of the method you want to ring"
     )
 
+    parser.add_argument(
+        "-b",
+        "--bob",
+        default="14",
+        help='An override for what place notation(s) should be made when a `Bob` is called in \
+              Ringing Room.  These will by default happen at the lead end.  Examples: "16" or \
+              "0:16" => 6ths place lead end bob.  "-1:3" or "-1:3.1" => a Grandsire Bob.  "20: 70" \
+              => a 70 bob taking effect 20 changes into a lead (the Half Lead for Surprise Royal). \
+              "20:7/0:4" => a 70 bob 20 changes into a lead and a 14 bob at the lead end. \
+              "3: 5/9: 5" => bobs in Stedman Triples.  Defaults to "14".'
+    )
+    parser.add_argument(
+        "-n",
+        "--single",
+        default="1234",
+        help='An override for what place notation(s) should be made when a `Single` is called in \
+              Ringing Room.  These will by default happen at the lead end.  Examples: "1678" or \
+              "0:1678" => 6ths place lead end single.  "-1:3.123" => a Grandsire Single. \
+              "20: 7890" => a 7890 single taking effect 20 changes into a lead (the Half Lead for \
+              Surprise Royal). "3: 567/9: 567" => singles in Stedman Triples.  Defaults to "1234".'
+    )
+
     args = parser.parse_args()
 
     # Run the program
     configure_logging()
 
-    tower = RingingRoomTower(args.room_id, get_load_balancing_url(args.room_id, args.url))
+    try:
+        socket_url = get_load_balancing_url(args.room_id, args.url)
+    except TowerNotFoundError as e:
+        sys.exit(f"Bad value for 'room_id': {e}")
+    except InvalidURLError as e:
+        sys.exit(f"Bad value for '--url': {e}")
+
+    tower = RingingRoomTower(args.room_id, socket_url)
     bot = Bot(tower, row_generator(args), args.use_up_down_in or args.handbell_style,
               args.stop_at_rounds or args.handbell_style, rhythm=rhythm(args))
 
