@@ -5,13 +5,13 @@ based on when the user-controlled bells ring.
 
 import logging
 import math
+import time
 
 from abc import ABCMeta, abstractmethod
-from time import sleep
 
 from wheatley.bell import Bell
 from wheatley.regression import calculate_regression
-
+from wheatley.tower import HANDSTROKE, BACKSTROKE
 
 WEIGHT_REJECTION_THRESHOLD = 0.001
 
@@ -72,11 +72,16 @@ class Rhythm(metaclass=ABCMeta):
 
         pass
 
+    def sleep(self, seconds: float):
+        """ Sleeps for given number of seconds. Allows mocking in tests"""
+        time.sleep(seconds)
+
 
 class WaitForUserRhythm(Rhythm):
     """ A decorator class that adds the ability to wait for user-controlled bells to ring. """
 
     logger_name = "RHYTHM:WaitForUser"
+    sleep_time = 0.01
 
     def __init__(self, rhythm: Rhythm):
         """
@@ -85,20 +90,24 @@ class WaitForUserRhythm(Rhythm):
         """
 
         self._inner_rhythm = rhythm
-        self._expected_bells = set()
+        self._current_stroke = HANDSTROKE
+        self._expected_bells = {HANDSTROKE: set(), BACKSTROKE: set()}
+        self._early_bells = {HANDSTROKE: set(), BACKSTROKE: set()}
         self.delay = 0
         self.logger = logging.getLogger(self.logger_name)
 
     def wait_for_bell_time(self, current_time, bell, row_number, place, user_controlled, stroke):
         """ Sleeps the thread until a given Bell should have rung. """
 
+        assert stroke == self._current_stroke
+
         self._inner_rhythm.wait_for_bell_time(current_time - self.delay, bell, row_number, place,
                                               user_controlled, stroke)
         if user_controlled:
             delay_for_user = 0
-            while (bell, stroke) in self._expected_bells:
-                sleep(0.01)
-                delay_for_user += 0.01
+            while bell in self._expected_bells[stroke]:
+                self.sleep(self.sleep_time)
+                delay_for_user += self.sleep_time
                 self.logger.debug(f"Waiting for {bell}")
             if delay_for_user:
                 self.logger.info(f"Delayed for {delay_for_user}")
@@ -113,18 +122,32 @@ class WaitForUserRhythm(Rhythm):
         """
 
         self._inner_rhythm.expect_bell(expected_bell, row_number, place, expected_stroke)
-        self._expected_bells.add((expected_bell, expected_stroke))
+
+        if expected_stroke != self._current_stroke:
+            self._current_stroke = expected_stroke
+            self._expected_bells[expected_stroke].clear()
+            self._early_bells[not expected_stroke].clear()
+
+        if expected_bell not in self._early_bells[expected_stroke]:
+            self._expected_bells[expected_stroke].add(expected_bell)
 
     def on_bell_ring(self, bell, stroke, real_time):
         """
         Called when a bell is rung at a given stroke.  Used as a callback from the Tower class.
         """
-
         self._inner_rhythm.on_bell_ring(bell, stroke, real_time - self.delay)
-        try:
-            self._expected_bells.remove((bell, stroke))
-        except KeyError:
-            pass
+
+        if stroke == self._current_stroke:
+            try:
+                self._expected_bells[self._current_stroke].remove(bell)
+            except KeyError:
+                pass
+            try:
+                self._early_bells[not self._current_stroke].remove(bell)
+            except KeyError:
+                pass
+        else:
+            self._early_bells[not self._current_stroke].add(bell)
 
     def initialise_line(self, stage, user_controls_treble, start_time,
                         number_of_user_controlled_bells):
@@ -208,19 +231,19 @@ class RegressionRhythm(Rhythm):
         if user_controlled and self._start_time == float('inf'):
             self.logger.debug(f"Waiting for pull off")
             while self._start_time == float('inf'):
-                sleep(0.01)
+                self.sleep(0.01)
             self.logger.debug(f"Pulled off")
             return
 
         bell_time = self.index_to_real_time(row_number, place)
         if bell_time == float('inf') or self._start_time == 0:
             self.logger.error(f"Bell Time {bell_time}; Start Time {self._start_time}")
-            sleep(self._blow_interval or 0.2)
+            self.sleep(self._blow_interval or 0.2)
         elif bell_time > current_time:
-            sleep(bell_time - current_time)
+            self.sleep(bell_time - current_time)
         else:
             # Slow the ticks slightly
-            sleep(0.01)
+            self.sleep(0.01)
 
     def expect_bell(self, expected_bell, row_number, place, expected_stroke):
         """
@@ -230,8 +253,8 @@ class RegressionRhythm(Rhythm):
         ringing.
         """
 
-        self.logger.debug(f"Expected bell {expected_bell} at index {row_number}:{place} at stroke" \
-                            + f"{expected_stroke}")
+        self.logger.debug(f"Expected bell {expected_bell} at index {row_number}:{place} at stroke"
+                          + f"{expected_stroke}")
         self._expected_bells[(expected_bell, expected_stroke)] = (row_number, place)
 
     def on_bell_ring(self, bell, stroke, real_time):
